/**
  ******************************************************************************
  * @file           : main.c
  * @author         ：SINOMCU-AE
  * @brief          : Main program body
  *    
   *          This file provides example code for ADC Trigger by Timer and printf by uart.
  * ADC IN1,IN2,IN3,IN4 interval 100ms trigger by Timer2 or Timer1,DMA Transfer,no filter.
  * Example use time out(USE_TIMEOUT 1),when error LED1 blink period 200mS by sysTick soft
  * timer,defualt blink period 1S;uart send information of running count(redirect printf).
  * For details,see "readme.txt".  
  *         
  ******************************************************************************
   * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, SinoMCU shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu All rights reserved.</center></h2>
  *
  ******************************************************************************
  */  
  
#include "system_define.h"

#define USER_VDDA                 3300           // unit mv
#define LED_BLINK_HALF_PER        500            // unit ms,LED blink half cycle

FlagStatus DMA_CH1_TC_Trigger = RESET;
uint16_t ADC_result_buff[ADC_BUFF_LEN]={0};

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void) {
  uint32_t count=0;
  ADCErrorStatus ADC_status = ADC_NO_ERROR;
  uint16_t voltage   = 0;

#if(defined(SYSCLK_USE_PLL_48M) && SYSCLK_USE_PLL_48M) 
  sysclk_init();                            // sysclk 48M,HSI 8M /2*12;if not call this function,default HSI 8M
#endif
  SystemCoreClockUpdate();                  // get System Core Clock

  sys_tick_init();
  gpio_init();
#if(defined(EN_PRINT_UART) && EN_PRINT_UART)   
  MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_HSI);  // uart clk HSI 8M      
  usart1_uart_init(115200);
#endif
  dma_ch1adc_init();
  adc_init();
  timer_init();

  LED1_ON(); 
  printf("\r\n\r\n*****ADC DMA Timer Trig Example*****");
  printf("\r\n--Inf:System Core Clock %d Hz.",SystemCoreClock);
  //user_config.h, 0:ADC TRIG TIM1 TRGO UIF,1:ADC TRIG TIM1 TRGO OC3REF,2:ADC TRIG TIM1 TRGO OC4REF 
#if( ADC_HW_TRIG == 0)       // user_config.h, 0:ADC TRIG TIM1 TRGO UIF   
  printf("\r\n--Inf:Timer1 Trigger UIF.");
#elif( ADC_HW_TRIG == 1)     // user_config.h, 1:ADC TRIG TIM1 TRGO OC3REF
  printf("\r\n--Inf:Timer1 Trigger OC3REF.");
#elif( ADC_HW_TRIG == 2)     // user_config.h, 2:ADC TRIG TIM1 TRGO OC4REF 
  printf("\r\n--Inf:Timer1 Trigger OC4REF.");
#elif( ADC_HW_TRIG == 3)     // user_config.h, 3:ADC TRIG TIM1 CH4
  printf("\r\n--Inf:Timer1 CH4 Trigger.");
#elif( ADC_HW_TRIG == 4)     // user_config.h, 4:ADC TRIG TIM2 TRGO
  printf("\r\n--Inf:Timer2 Trigger.");
#else
#error user_config.h,ADC Hardware Tigger select Error!!!
#endif
  
  ADC_status = Activate_ADC();
  if(ADC_status !=ADC_NO_ERROR ) {
    ADC_ERROR(ADC_status);
  }
  MS32_ADC_REG_StartConversion(ADC1);                           // Hardware Trigger need 

  Start_User_MS_SF_Timer(LED_BLINK_HALF_PER);
  while(1) {
    if(DMA_CH1_TC_Trigger == SET) {                             // DMA interrupt callback set flag,DMA transfer one time finish
      DMA_CH1_TC_Trigger = RESET;
      voltage = __MS32_ADC_CALC_DATA_TO_VOLTAGE(USER_VDDA, ADC_result_buff[0], MS32_ADC_RESOLUTION_12B);
      printf("\r\nADC PA1: %d mV", voltage);
      voltage = __MS32_ADC_CALC_DATA_TO_VOLTAGE(USER_VDDA, ADC_result_buff[1], MS32_ADC_RESOLUTION_12B);
      printf(",PA2: %d mV", voltage);
      voltage = __MS32_ADC_CALC_DATA_TO_VOLTAGE(USER_VDDA, ADC_result_buff[2], MS32_ADC_RESOLUTION_12B);
      printf(",PA3: %d mV", voltage);
      voltage = __MS32_ADC_CALC_DATA_TO_VOLTAGE(USER_VDDA, ADC_result_buff[3], MS32_ADC_RESOLUTION_12B);
      printf(",PA4: %d mV", voltage);
    }
   
    if(Get_State_User_MS_SF_Timer() == SF_TIMER_OV) {
      Start_User_MS_SF_Timer(LED_BLINK_HALF_PER);               // clear SF_TIMER_OV
      LED1_TOGGLE();
      count++;
      printf("\r\n--Inf: running count:%d",count);
    }
  }
}

/**
  * @brief  SystemClock_Config HSI/2 to PLL*12 sysclk 48M
  * @param  None
  * @retval None
  */
void sysclk_init(void) {
  MS32_RCC_OscInitTypeDef osc_cfg;
  MS32_RCC_PllInitTypeDef pll_cfg;
  MS32_RCC_ClkInitTypeDef clk_cfg;

  // get and clear reset flag 
  MS32_RCC_GetResetFlag(ENABLE);
  
  // osc config 
  osc_cfg.HSIState = MS32_RCC_HSI_ON;
  osc_cfg.HSEState = MS32_RCC_HSE_OFF;
  osc_cfg.LSIState = MS32_RCC_LSI_OFF;
//  osc_cfg.LSEState = MS32_RCC_LSE_ON_DRV_H;
  MS32_RCC_OscConfig(&osc_cfg);
  
  // pll config 
  pll_cfg.PLLState  = MS32_RCC_PLL_ON;
  pll_cfg.PLLSource = MS32_RCC_PLL_SOURCE_HSI_DIV_2;            // HSI/2 4M
  pll_cfg.PLLMUL    = MS32_RCC_PLL_MUL12;
  pll_cfg.PREDIV    = MS32_RCC_PLL_PREDIV_DIV1;
  MS32_RCC_PllConfig(&pll_cfg);
  
  // sysclk config 
  clk_cfg.SYSCLKSource   = MS32_RCC_SYSCLK_SOURCE_PLL;
  clk_cfg.AHBCLKDivider  = MS32_RCC_SYSCLK_DIV1;
  clk_cfg.APB1CLKDivider = MS32_RCC_HCLK_DIV1;
  MS32_RCC_ClockConfig(&clk_cfg);
}

/******************************** END OF FILE *********************************/
