/**
  ******************************************************************************
  * @file     TIMER_CFG.c
  * @author   SINOMCU-AE
  * @brief    Timer basic config
  *    
  *          This file provides functions to Timer basic config:
  *             Timer1 or 2 set 100ms,for ADC Tigger.
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, SinoMCU shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu All rights reserved.</center></h2>
  *
  ******************************************************************************
  */
#include "user_config.h"    
#include "TIMER_CFG.h"
/**
  * @brief  Timer Initialization for ADC Trigger Function
  * @param  None
  * @retval None
  */
void timer_init(void) {
  MS32_TIM_InitTypeDef TimerInitStruct = {0};
#if (( ADC_HW_TRIG == 1) ||( ADC_HW_TRIG == 2) ||( ADC_HW_TRIG == 3))
  MS32_TIM_OC_InitTypeDef TimerOCInitStruct = {0};
#endif
  
  // timer 100ms,Trig to ADC
  MS32_TIM_StructInit(&TimerInitStruct);
  TimerInitStruct.CounterMode =MS32_TIM_COUNTERMODE_UP;
  TimerInitStruct.Prescaler   = __MS32_TIM_CALC_PSC(SystemCoreClock,1000);                         // unit Hz,div sysclk to  CNT clk:1khz           
  TimerInitStruct.Autoreload  = __MS32_TIM_CALC_ARR(SystemCoreClock,TimerInitStruct.Prescaler,10); // unit Hz,10Hz(100ms)
  //user_config.h, 0:ADC TRIG TIM1 TRGO UIF,1:ADC TRIG TIM1 TRGO OC3REF,2:ADC TRIG TIM1 TRGO OC4REF,3:ADC TRIG TIM1 CH4
#if(( ADC_HW_TRIG == 0) ||( ADC_HW_TRIG == 1) ||( ADC_HW_TRIG == 2) ||( ADC_HW_TRIG == 3))         //Timer1
  TimerInitStruct.RepetitionCounter = 0;                  
  MS32_TIM_Init(TIM1, &TimerInitStruct);                   // init function had enable clock

  MS32_TIM_SetClockSource(TIM1, MS32_TIM_CLOCKSOURCE_INTERNAL);           // clk form RCC:PCLK <-- AHP_RRE=1,sysclk <-- HSI8M or PLL
  MS32_TIM_EnableARRPreload(TIM1);
  MS32_TIM_DisableMasterSlaveMode(TIM1);                                  // disable the Master/Slave mode.

 #if( ADC_HW_TRIG == 0)                                                   // user_config.h, 0:ADC TRIG TIM1 TRGO UIF
  MS32_TIM_SetTriggerOutput(TIM1, MS32_TIM_TRGO_UPDATE); 
 #elif (( ADC_HW_TRIG == 1) ||( ADC_HW_TRIG == 2) ||( ADC_HW_TRIG == 3))  // user_config.h, 1,2,3
  MS32_TIM_OC_StructInit(&TimerOCInitStruct);
  TimerOCInitStruct.OCMode   = MS32_TIM_OCMODE_PWM1;                      // PWM1 mode
  TimerOCInitStruct.CompareValue =  ((( MS32_TIM_GetAutoReload(TIM1) + 1 ) * 25 ) / 100);     // set duty cycle 25%

  #if ( ADC_HW_TRIG == 1)                                                 // user_config.h, 1:ADC TRIG TIM1 TRGO OC3REF
  MS32_TIM_OC_Init(TIM1, MS32_TIM_CHANNEL_CH3, &TimerOCInitStruct); 
  MS32_TIM_SetTriggerOutput(TIM1, MS32_TIM_TRGO_OC3REF); 
  #elif ( ADC_HW_TRIG == 2)                                               // user_config.h, 2:ADC TRIG TIM1 TRGO OC4REF
  MS32_TIM_OC_Init(TIM1, MS32_TIM_CHANNEL_CH4, &TimerOCInitStruct); 
  MS32_TIM_SetTriggerOutput(TIM1, MS32_TIM_TRGO_OC4REF);                  
  #else  // ADC_HW_TRIG == 3                                              // user_config.h, 3:ADC TRIG TIM1 CH4
  MS32_TIM_OC_Init(TIM1, MS32_TIM_CHANNEL_CH4, &TimerOCInitStruct);
  MS32_TIM_CC_EnableChannel(TIM1, MS32_TIM_CHANNEL_CH4);                  // enable channel,ADC_TIMER_TIM1_CH4
  MS32_TIM_EnableAllOutputs(TIM1);                                        // enable ouput,ADC_TIMER_TIM1_CH4 
  #endif 
 #endif
  MS32_TIM_EnableCounter(TIM1);                                           // start timer 
#elif( ADC_HW_TRIG == 4)                                                  // user_config.h, 4:ADC TRIG TIM2 TRGO;Timer2
  MS32_TIM_Init(TIM2, &TimerInitStruct); 

  MS32_TIM_SetClockSource(TIM2, MS32_TIM_CLOCKSOURCE_INTERNAL);           // clk form RCC:PCLK <-- AHP_RRE=1,sysclk <-- HSI8M or PLL
  MS32_TIM_EnableARRPreload(TIM2);
  MS32_TIM_DisableMasterSlaveMode(TIM2);                                  // disable the Master/Slave mode.
  MS32_TIM_SetTriggerOutput(TIM2, MS32_TIM_TRGO_UPDATE);                  // Trig out 
  
  MS32_TIM_EnableCounter(TIM2);                                           // start timer
#else
#error user_config.h,ADC Hardware Tigger select Error!!!
#endif
}

/******************************** END OF FILE *********************************/
