/**
  ******************************************************************************
  * @file     USART1_CFG.c
  * @author   SINOMCU-AE
  * @brief    USART1 config
  *    
  *          This file provides functions to usart1 initialization:
  *              PA9   --> USART1_TX
  *              PA10  <-- USART1_RX
  *          BaudRate:115200,8 bit , NONE,STOP 1bit
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, SinoMCU shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu All rights reserved.</center></h2>
  *
  ******************************************************************************
  */
#include "stdio.h"
#include "user_config.h"  
#include "USART1_CFG.h"

#if defined ( __GNUC__ ) && !defined (__clang__)      // GCC, not Keil MDK AC6
/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  fd:file descriptor,ptr:send buff pointer,len:send byte number
  * @retval None
  */
int _write (int fd, char *ptr, int len) {
#if(defined(EN_PRINT_UART) && EN_PRINT_UART)  
  uint16_t send_data_len=len;
  for(; send_data_len >0;) {
    if(MS32_USART_IsActiveFlag_TXE(USART1)) {
      MS32_USART_TransmitData8(USART1,*ptr++);
      send_data_len--;
    }
  }
#endif
  return len;
}  
#endif
#if defined ( __CC_ARM ) || defined (__clang__)       // Keil MDK, AC5|| AC6
/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  ch:send byte,file:library,file pointer
  * @retval None
  */
int fputc(int ch, FILE *f) {
#if(defined(EN_PRINT_UART) && EN_PRINT_UART)  
  while(!(MS32_USART_IsActiveFlag_TXE(USART1))){ ; }  // wait TXE set
  MS32_USART_TransmitData8(USART1,ch);
#endif
  return ch;
}
#endif
#ifdef __ICCARM__  // IAR 
/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  ch:send byte
  * @retval None
  */ 
int putchar(int ch) {
#if(defined(EN_PRINT_UART) && EN_PRINT_UART)  
  while(!(MS32_USART_IsActiveFlag_TXE(USART1))){ ; }  // wait TXE set
  MS32_USART_TransmitData8(USART1,ch);
#endif
  return ch;
}
#endif

/**
  * @brief  uart IO Initialization
  *    PA9   --> USART1_TX
  *    PA10  <-- USART1_RX
  * @param  None
  * @retval None
  */
void uart_gpio_init(void) {
  MS32_GPIO_InitTypeDef GpioInitStruct = {0};

  MS32_GPIO_StructInit(&GpioInitStruct);    
  GpioInitStruct.Pin = MS32_GPIO_PIN_9 | MS32_GPIO_PIN_10;
  GpioInitStruct.Mode = MS32_GPIO_MODE_ALTERNATE;
  GpioInitStruct.Speed = MS32_GPIO_SPEED_FREQ_HIGH;
  GpioInitStruct.OutputType = MS32_GPIO_OUTPUT_PUSHPULL;
  GpioInitStruct.Pull = MS32_GPIO_PULL_UP;
  GpioInitStruct.Alternate = MS32_GPIO_AF_1;
  MS32_GPIO_Init(GPIOA, &GpioInitStruct);             // init function had enable clock
}

/**
  * @brief  uart1 Initialization Function
  * @param  uint32_t baudrate
  * @retval None
  */
void usart1_uart_init(uint32_t baudrate) {
  MS32_USART_InitTypeDef UsartInitStruct = {0};

  uart_gpio_init();
  
  // UART parameter configuration
  MS32_USART_StructInit(&UsartInitStruct);
  UsartInitStruct.BaudRate = baudrate;
  UsartInitStruct.DataWidth = MS32_USART_DATAWIDTH_8B;
  UsartInitStruct.StopBits = MS32_USART_STOPBITS_1;
  UsartInitStruct.Parity = MS32_USART_PARITY_NONE;
  UsartInitStruct.TransferDirection = MS32_USART_DIRECTION_TX_RX;
  UsartInitStruct.HardwareFlowControl = MS32_USART_HWCONTROL_NONE;
  UsartInitStruct.OverSampling = MS32_USART_OVERSAMPLING_16;
  MS32_USART_Init(USART1, &UsartInitStruct);          // init function had enable clock
  
  MS32_USART_DisableIT_CTS(USART1);
  MS32_USART_ConfigAsyncMode(USART1);
  MS32_USART_Enable(USART1);
}

/******************************** END OF FILE *********************************/
